#pragma once

#include <map>
#include <mutex>

#include "accessor.h"
#include "pager.h"
#include "page_mapper.h"



/* Mapping type from accessors to page mappers. */

typedef std::map<fileid_t, PageMapper *> FileMapping;
typedef std::pair<fileid_t, PageMapper *> FileMappingEntry;



/* A registry of mappers for accessors. */

class FilePaging
{
protected:
    Pages *_pages;

    FileMapping _mappers;
    std::mutex _lock;

    /* Pager initialisation methods. */

    PageMapper *get_mapper(fileid_t fileid);

    Pager *get_pager(fileid_t fileid, flags_t flags);

    /* Configurable methods. */

    virtual fileid_t get_fileid(const char *path) = 0;

    virtual flags_t get_flags(flags_t flags);

    virtual Accessor *make_accessor(fileid_t fileid) = 0;

    /* Mapper registry access. */

    PageMapper *get(fileid_t fileid);

    void set(fileid_t fileid, PageMapper *mapper);

public:
    explicit FilePaging(Pages *pages);

    virtual ~FilePaging()
    {
    }

    /* Methods for the pager. */

    void detach_pager(fileid_t fileid, PageMapper *mapper);
};

// vim: tabstop=4 expandtab shiftwidth=4
